#define YMW_CURRENT_VERSION    1    /* Version number */
#define YMW_CURRENT_REVISION   16   /* Revision number */

#define YFMUDED_REL_TYPE       "Public Beta"   /* What kind of release */
#define YFMUDED_REL_VER        "0.10"          /* What release version is it */

/* Content types
 ***/
#define CONT_LOAD_MOBILE                0
#define CONT_LOAD_OBJECT                1
#define CONT_GIVE_OBJECT_TO_MOBILE      2
#define CONT_EQUIP_MOBILE_WITH_OBJECT   3
#define CONT_PUT_OBJECT_IN_OBJECT       4
#define CONT_SET_DOOR_STATE             5
#define CONT_REMOVE_OBJECT_FROM_ROOM    6

/* Directional info, used mainly for doors
 ***/
#define D_NORTH      0    /* North */
#define D_EAST       1    /* East */
#define D_SOUTH      2    /* South */
#define D_WEST       3    /* West */
#define D_UP         4    /* Up */
#define D_DOWN       5    /* Down */

/* Door flags, keys, 1-way, etc.
 ***/
#define DOOR_PASSAGE 0    /* Empty passageway */
#define D_CANT_LEAVE 1    /* If this is set than you can't use the door to leave from the room that it's set in */

/* Handy numbers that may change, but are inherent in the system.
 * Not user settable!
 ***/
#define ND           6    /* Number of doors */

/* Map-Editor modes, for use with other misc. tools.
 ***/
#define EM_NORM      1    /* Normal editing */
#define EM_DPICK     2    /* Re-assign a door mode */
#define EM_DPADD     3    /* Door add mode (select the seccond room) */

/* This thing is extra components to be a part of all dialogs, like
 * taking screen-shots and getting help and stuff.
 ***/
#define standarnd_dialog_chunks \
      { d_keyboard_proc,         0,   0,   0,   0,   0,   0,   0,   0,      KEY_F2,     0,  scrn_shot,               NULL,     NULL }, \
      { NULL,                    0,   0,   0,   0,   0,   0,   0,   0,      0,          0,  NULL,                    NULL,     NULL }

/* This is an experimental "greyer" to help focus on the active dialog
 ***/
#define greyer_thing_code                                        \
    if( msg == MSG_START ){                                      \
      int x, y;                                                  \
                                                                 \
      scare_mouse();                                             \
      for( x = 0; x < SCREEN_W; x++ ){                           \
         for( y = 0; y < SCREEN_H; y++ ){                        \
            if( (x+(y%2))%2 )                                    \
              putpixel( screen, x, y, makecol(128,128,128) );    \
            }                                                    \
         }                                                       \
      unscare_mouse();                                           \
      }

/* These are global variables that must be accessable from every file
 ***/
extern struct WORLD world;       /* Main world var, contains everything */
extern int cur_room;             /* Which room is selected, -2 for Multi-Sel */
extern int edmode;               /* Mode of the room-editor */
extern int cur_dir;              /* Which dir is selected in the door-editor */
extern int odir;                 /* Other dir, from r2 in the door-editor */
extern int cur_zone;             /* Which zone we're adding rooms to */
extern int cur_layer;            /* Like the current Z, but global (effects everything ) */
extern int cur_object;           /* Which object we're editing in this zone */
extern int cur_mobile;           /* Which mobile we're editing in this zone */
extern int cur_shop;             /* Which shop we're editing in this zone */
extern char blankstr[1];         /* Blank string */
extern unsigned short mapopt;    /* Map Drawing Options */
extern char filename[0x400];     /* The current file that's being worked on */
extern struct DATAFILE *dat;     /* A datafile with stuff in it! */
extern struct BOARD board;       /* The clipboard */
extern char username[80];        /* The guy's (gal's?) name */
extern struct MSEL msel_list;    /* The msel list, the list of all selected rooms (others?) */

typedef struct FULL_VNUM
{
    int zone, room;       /* The zone and room that this vnum points to */
} FULL_VNUM;

typedef struct DOOR
{
    int croom;            /* Room that this door Connects to */
    int czone;            /* Zone that this door Connects to */
    int cdir;             /* Direction that this door Connects to */
    short typea;          /* Flags for this door, source end (clear, door, pick-proof) */
    short typeb;          /* Flags for this door, source end (twoway, oneway-out, oneway-in) */
    int key;              /* Key for this door */
    int state;            /* State of the door */
    int dir;              /* Direction this door leads away fromt the source room in */
    char *desc;           /* Description for the source room */
    char *keywords;       /* Keywords for the source room */

    struct DOOR *other;   /* A link to the other door */
    struct ROOM *parent;  /* A link to the parent room */
} DOOR;

typedef struct EXD        /* Extra Description */
{
    char *keywords;       /* Keyword list for this description */
    char *desc;           /* Description */
} EXD;

typedef struct RCONTENT   /* Room Content */
{
    int type;             /* Which kind is it? */
    int vnum;             /* The vnum of whatever it is we're doing (vnum1) */
    int ifflag;           /* The magical if-flag */
    int max;              /* Maximum of these to exist for this to work */
    int extra;            /* One extra (depends on type) */
} RCONTENT;

typedef struct ROOM       /* Data for each room */
{
    int x, y, z;          /* Position x, y, & z */
    char *title;          /* Title of room */
    char *desc;           /* Full description of room */
    int vnum;             /* Preset VNUM?  -1 for no */
    int zone;             /* Which zone? */
    unsigned long long rbv; /* Room BitVector */
    int stype;            /* Sector type */

    int num_doors;        /* Number of doors */
    DOOR *door;           /* Door data, n,e,s,w,u,d */

    int num_exd;          /* Number of extra descriptions */
    EXD *exd;             /* Extra descriptions */

    int num_cont;         /* Number of contained items */
    RCONTENT *content;    /* Content data */

    struct ZONE *parent;  /* A link to the parent zone */
} ROOM;

typedef struct OAFFECT
{
    int location;         /* What it affects */
    int value;            /* How much it affects */
} OAFFECT;

typedef struct OBJECT
{
    int vnum;             /* Preset VNUM?  -1 for no */
    char *keywords;       /* Keyword list for this object */
    char *short_desc;     /* Short Description */
    char *long_desc;      /* Long Description */
    char *action_desc;    /* Action Description */
    unsigned int type;    /* Type flag */
    unsigned long long ebv; /* Extra (Effects) BitVector */
    unsigned long long wbv; /* Wear BitVector */
    int value[4];         /* Value definitions */
    int weight;           /* How heavy this thing is */
    int cost;             /* The base value of this object */
    int rpd;              /* The rent-per-day value of this object */
    int num_exd;          /* Number of extra descriptions */
    EXD *exd;             /* Extra descriptions */
    int num_affect;       /* Number of Object Affect Fields */
    OAFFECT *affect;      /* Object Affect Field data */

    struct ZONE *parent;  /* A link to the parent zone */
} OBJECT;

typedef struct ESPEC
{
    int type;             /* What kind is it */
    int value;            /* What value does it have */
} ESPEC;

typedef struct MOBILE
{
    int vnum;             /* Preset VNUM?  -1 for no */
    char *keywords;       /* Keyword list for this object */
    char *short_desc;     /* Short Description */
    char *long_desc;      /* Long Description */
    char *detail_desc;    /* Detailed Description */
    unsigned long long actbv; /* Action BitVector */
    unsigned long long affbv; /* Affection BitVector */
    int alignment;        /* Allignment; -1000 to 1000 */
    char typeflag;        /* Type Flag, either S or E */
    int level;            /* Level of mobile */
    int thac0;            /* To Hit Armor Class 0; 0 to 20 */
    int ac;               /* Armor Class; -10 to 10 */
    int mhpx, mhpy, mhpz; /* Max HP in format xdy+z (x dice sized y plus z) */
    int bhdx, bhdy, bhdz; /* Bare Hand Damage in format xdy+z */
    int gold;             /* Amount of gold mobile is born with */
    int exp;              /* Experiance points born with */
    int loadpos;          /* Load position */
    int defpos;           /* Default position */
    char sex;             /* Neuter, Male, Female */
    int num_especs;       /* Number of E-Specs */
    ESPEC *espec;         /* E-Spec data */

    struct ZONE *parent;  /* A link to the parent zone */
} MOBILE;

typedef struct TAG
{
    int x, y, z;          /* Position */
    int w, h;             /* Droppy Dealies */
    char *str;            /* Text to place */
    struct ZONE *parent;  /* A link to the parent zone */
} TAG;

typedef struct BSPEC
{
    int type;             /* What kind of thing will be bought (from items) */
    char *namelist;       /* The boolean namelist */
} BSPEC;

typedef struct SHOP
{
    int vnum;             /* Virtual number of shop, index */

    int num_items;        /* Number of items in the shop */
    int *item;            /* The item-list (vnums) */

    float prof_sell;      /* Profit when selling */
    float prof_buy;       /* Profit when buying */

    int num_bspecs;       /* Number of buy-specs */
    BSPEC *bspec;         /* bspec data */

    /* The following are from the players point of view, buy means the player
     * is buying somthing from the shop, so the first would be if the shop
     * doesn't carry what the player wants to buy from them.
     ***/
    char *buy_item_ne;   /* Message when item to buy does not exist */
    char *sell_item_ne;  /* Message when item to sell does not exist */
    char *refuse_offer;  /* Message when shop does not buy offered item */
    char *shop_afford;   /* Message when shop can't afford item */
    char *play_afford;   /* Message when player can't afford item */
    char *buy_success;   /* Message when successfully buying an item */
    char *sell_success;  /* Message when successfully selling an item */

    int temper;          /* What the shop does when the player can't afford an item */
    long long shop_bv;   /* Shop BitVector */
    int mobile;          /* The mobile to link to */
    long long withwho_bv;/* Trade With Who BitVector */

    int num_rooms;       /* The number of rooms this shop can trade in */
    int *room;           /* The list of rooms (vnums) this shop can trade in */

    int topen[2];        /* Times the shop opens at */
    int tclose[2];       /* Times the shop closes at */

    struct ZONE *parent; /* A link back to the parent zone */
} SHOP;

typedef struct ZONE
{
    char *name;           /* Name of the zone */
    char *author;         /* Name of author, or whoever gets credit */
    int tint;             /* The color tint for rooms in this zone */
    int vnum;             /* Preset VNUM?  -1 for no */
    int life_span;        /* Number of minuets between resets */
    int reset_mode;       /* Never, deserted, always */
    unsigned long long rbv; /* Default Room BitVector for this zone */
    int stype;            /* Default Sector type for this zone */

    int num_rooms;        /* Number of rooms */
    ROOM *room;           /* Room data */
    int last_room_vnum;   /* The last vnum issued (for rooms) */

    int num_objects;      /* Number of objects in this zone */
    OBJECT *object;       /* Object data */
    int last_object_vnum; /* The last vnum issued (for objects) */

    int num_mobiles;      /* Number of mobiles in this zone */
    MOBILE *mobile;       /* Mobile data */
    int last_mobile_vnum; /* The last vnum issued (for mobiles) */

    int num_shops;        /* Number of shops in this zone */
    SHOP *shop;           /* Shop data */
    int last_shop_vnum;   /* The last vnum given to a shop */

    int num_tags;         /* Number of tags */
    TAG *tag;             /* Tag data */

    struct WORLD *parent; /* A link to the parent world */
} ZONE;

typedef struct WORLD      /* World data, keeps track of globals & subsets */
{
    int num_zones;        /* Number of zones */
    ZONE *zone;           /* Zone data */
    int last_zone_vnum;   /* The last vnum issued (for zones) */
} WORLD;

typedef struct BVLIST     /* Special structure to store a list of bits in a "BitVector" */
{
    char *desc;           /* The textual description for use in lists */
    int bitpos;           /* The position of the bit to be turned on (in expression (1<<bitpos)) */
} BVLIST;

typedef struct BOARD      /* Data for the clipboard */
{
    int type;
    void *data;
} BOARD;

typedef struct MSEL_ITEM  /* The read data for multiple_select_dealies */
{
    int hi_ind;             /* The zone (vnum?) */
    int lo_ind;             /* ..and room(?) (vnum?) */
    int x, y;               /* The x & y (relative for moving) */
    struct MSEL_ITEM *next; /* The next item */
} MSEL_ITEM;

typedef struct MSEL       /* Data for multiple-selections */
{
    int num_sel;          /* How many things are selected */
    MSEL_ITEM *base;      /* The base item */
} MSEL;
